#include "include.h"
#include "config.h"
#include "log.h"
#include "wifi_clients.h"
#include "ubus.h"

#ifndef MINI
#include "socket.h"
#endif

char *ifname = "br-wan";
char *addr = "::";
char *maddr = "ff02::31f1";
char *port = "1000";

struct option longopts[] = {
	{"verbose", no_argument, 0, 'v'},
	{"help", no_argument, 0, 'h'},
	// Bandsteering options
	{"try-probe", required_argument, 0, 'p'},
	{"try-auth", required_argument, 0, 'a'},
	{"signal", required_argument, 0, 's'},
	{"force-auth", optional_argument, 0, 'f'},
	{"force-probe", optional_argument, 0, 1},
	{"learning-probe", optional_argument, 0, 2},
	// memory options
	{"clean-every", required_argument, 0, 3},
	{"clean-older-then", required_argument, 0, 4},
	{"clean-authed", optional_argument, 0, 5},
	#ifndef MINI
	// network options
	{"addr", required_argument, 0, 6},
	{"maddr", required_argument, 0, 7},
	{"port", required_argument, 0, 8},
	{"if", required_argument, 0, 9},
	{"mif", required_argument, 0, 10},
	// other extended option
	{"learning-socket", optional_argument, 0, 11},
	{"socket-always-answer", optional_argument, 0, 12},
	#endif
};

void usage(int c) {
	printf("Usage: wifictld ...\n");
	#ifdef DEBUG
	printf(" -v|--verbose         log verbose and debug (current: %s)\n", BOOL2STR(config_verbose));
	#else
	printf(" -v|--verbose         log verbose (current: %s)\n", BOOL2STR(config_verbose));
	#endif
	printf(" -h|--help            show this help text\n");
	printf("\nbandsteering options:\n");
	printf(" -p|--try-probe       threashold of wifi probe till allow (current: %d)\n", config_client_try_probe_threashold);
	printf(" -a|--try-auth        threashold of wifi auth till allow (current: %d)\n", config_client_try_auth_threashold);
	printf(" -s|--signal          threashold to allow (current: %d)\n", config_client_signal_threashold);
	printf(" -f|--force-auth           force steering (current: %s)\n", BOOL2STR(config_client_force_auth));
	printf(" --force-probe        force steering only for probes (current: %s)\n", BOOL2STR(config_client_force_probe));
	printf(" --learning-probe     probe-learing (current: %s)\n", BOOL2STR(config_client_learning_probe));
	printf("                      (parameter ignored if probe-steering on)\n");
	printf("\nmemory cleanup options:\n");
	printf(" --clean-every        run cleaning every (current: %ds)\n", config_client_clean_every);
	printf(" --clean-older-then   clean unseen for secound (current: %ds)\n", config_client_clean_older_then);
	printf(" --clean-authed       clean already authed also (not only learned by probes) (current: %s)\n", BOOL2STR(config_client_clean_authed));
	#ifndef MINI
	printf("\nnetwork options:\n");
	printf(" --addr                   listen on ip address (current: %s)\n", addr);
	printf(" --maddr                  listen and request on multcast address (current: %s)\n", maddr);
	printf(" --port                   listen on port (current: %s)\n", port);
	printf(" --if                     listen on interface (current: %s)\n", ifname);
	printf(" --mif                    listen on interface for multicast (current: %s)\n", ifname);
	printf("\nother extended options:\n");
	printf(" --learning-socket        learn clients during listen on socket  (current: %s)\n", BOOL2STR(config_client_learning_socket));
	printf(" --socket-always-answer   answer on socket only with useful information  (current: %s)\n", BOOL2STR(config_socket_always_answer));
	#endif
	printf("\nSome config could be changed by ubus, see `ubus -v list wifictld`\n");
	if(c)
		printf("Invalid parameter %c ignored.\n", c);
}

int main(int argc, char *argv[])
{
	int ret = 0;

	#ifdef MINI
	log_info("start wifictld (mini)\n");
	#else
	log_info("start wifictld (full)\n");
	#endif

	int c = 0, indexptr = 0;
	while ((c = getopt_long(argc, argv, "vh:pas:f",longopts, &indexptr)) != -1) {
		switch (c) {
			case 'v':
				config_verbose = !config_verbose;
				break;
			case 'h':
				usage(0);
				return 0;
			// Bandsteering options
			case 'p':
				config_client_try_probe_threashold = atoi(optarg);
				if (config_client_try_probe_threashold < 0) {
					printf("negativ tries count not supported\n");
					return 1;
				}
				break;
			case 'a':
				config_client_try_auth_threashold = atoi(optarg);
				if (config_client_try_auth_threashold < 0) {
					printf("negativ tries count not supported\n");
					return 1;
				}
				break;
			case 's':
				config_client_signal_threashold = atoi(optarg);
				if (config_client_signal_threashold > 0) {
					printf("prositiv signal not supported\n");
					return 1;
				}
				break;
			case 'f':
				if(optarg)
					config_client_force_auth = atoi(optarg);
				else
					config_client_force_auth = !config_client_force_auth;
				break;
			case 1:
				if(optarg)
					config_client_force_probe = atoi(optarg);
				else
					config_client_force_probe = !config_client_force_probe;
				break;
			case 2:
				if(optarg)
					config_client_learning_probe = atoi(optarg);
				else
					config_client_learning_probe = !config_client_learning_probe;
				break;
			// memory options
			case 3:
				config_client_clean_every = atoi(optarg);
				if (config_client_clean_every < 0) {
					printf("negativ time not supported\n");
					return 1;
				}
				break;
			case 4:
				config_client_clean_older_then = atoi(optarg);
				if (config_client_clean_older_then < 0) {
					printf("negativ time not supported\n");
					return 1;
				}
				break;
			case 5:
				if(optarg)
					config_client_clean_authed = atoi(optarg);
				else
					config_client_clean_authed = !config_client_clean_authed;
				break;
			#ifndef MINI
			// network options
			case 6:
				addr = optarg;
				break;
			case 7:
				maddr = optarg;
				break;
			case 8:
				port = optarg;
				break;
			case 9:
				ifname = optarg;
				break;
			case 10:
				// skip - check later after init socket
				break;
			// other extended option
			case 11:
				if(optarg)
					config_client_learning_socket = atoi(optarg);
				else
					config_client_learning_socket = !config_client_learning_socket;
				break;
			case 12:
				if(optarg)
					config_socket_always_answer = atoi(optarg);
				else
					config_socket_always_answer = !config_socket_always_answer;
				break;
			#endif
			default:
				usage(c);
				return 1;
		}
	}

	log_debug("log: show debug\n");
	log_verbose("log: show verbose\n");

	uloop_init();

	// define wifi clients memory
	ret = wifi_clients_init();
	if (ret)
	{
		log_error("exit with error on client init\n");
		return ret;
	}
	// bind to loop
	ret = wifictld_ubus_init();
	if (ret)
	{
		wifi_clients_close();
		log_error("exit with error on ubus init\n");
		return ret;
	}
	#ifndef MINI
	ret = socket_init(ifname, addr, maddr, port);
	if (ret)
	{
		socket_close();
		log_error("exit with error on socket init\n");
		return ret;
	}
	c = 0;
	indexptr = 0;
	while ((c = getopt_long(argc, argv, "vh:pas:f",longopts, &indexptr)) != -1) {
		switch (c) {
			case 10:
			socket_join_mcast(optarg);
			break;
		}
	}
	#endif
	uloop_run();
	uloop_done();


	wifictld_ubus_close();
	wifi_clients_close();

	log_info("safe exit\n");

	return 0;
}
